Equip Board Options dialog with themes listbox
[xboard.git] / dialogs.c
index 6fa9065..df53d8a 100644 (file)
--- a/dialogs.c
+++ b/dialogs.c
@@ -846,20 +846,20 @@ SoundOptionsProc ()
 
 static void DefColor P((int n));
 static void AdjustColor P((int i));
+static void ThemeSel P((int n, int sel));
+static int BoardOptionsOK P((int n));
 
 static char oldPieceDir[MSG_SIZ];
+extern char *engineLine, *nickName; // defined later on
 
-static int
-BoardOptionsOK (int n)
-{
-    if(appData.overrideLineGap >= 0) lineGap = appData.overrideLineGap; else lineGap = defaultLineGap;
-    InitDrawingParams(strcmp(oldPieceDir, appData.pieceDirectory));
-    InitDrawingSizes(-1, 0);
-    DrawPosition(True, NULL);
-    return 1;
-}
+#define THEMELIST 1
 
 static Option boardOptions[] = {
+{   0,LR|T2T, 0, NULL, NULL, NULL, NULL, Label, N_("Selectable themes:") },
+{ 300,LR|TB,200, NULL, (void*) engineMnemonic, (char*) &ThemeSel, NULL, ListBox, "" },
+{   0,LR|T2T, 0, NULL, NULL, NULL, NULL, Label, N_("New name for current theme:") },
+{ 0, 0, 0, NULL, (void*) &nickName, ".png", NULL, TextBox, "" },
+{ 0,SAME_ROW, 0, NULL, NULL, NULL, NULL, Break, NULL },
 { 0,          0, 70, NULL, (void*) &appData.whitePieceColor, "", NULL, TextBox, N_("White Piece Color:") },
 { 1000, SAME_ROW, 0, NULL, (void*) &DefColor, NULL, (char**) "#FFFFCC", Button, "      " },
 /* TRANSLATORS: R = single letter for the color red */
@@ -913,6 +913,16 @@ static Option boardOptions[] = {
 { 0, 0, 0, NULL, (void*) &BoardOptionsOK, "", NULL, EndMark , "" }
 };
 
+static int
+BoardOptionsOK (int n)
+{
+    if(n && (n = SelectedListBoxItem(&boardOptions[THEMELIST])) > 0 && *engineList[n] != '#') { // called by pressing OK, and theme selected
+       ASSIGN(engineLine, engineList[n]);
+    }
+    LoadTheme();
+    return 1;
+}
+
 static void
 SetColorText (int n, char *buf)
 {
@@ -956,9 +966,32 @@ AdjustColor (int i)
 }
 
 void
+ThemeSel (int n, int sel)
+{
+    int nr;
+    char buf[MSG_SIZ];
+    if(sel < 1) buf[0] = NULLCHAR; // back to top level
+    else if(engineList[sel][0] == '#') safeStrCpy(buf, engineList[sel], MSG_SIZ); // group header, open group
+    else { // normal line, select engine
+       ASSIGN(engineLine, engineList[sel]);
+       LoadTheme();
+        PopDown(TransientDlg);
+       return;
+    }
+    nr = NamesToList(appData.themeNames, engineList, engineMnemonic, buf); // replace list by only the group contents
+    ASSIGN(engineMnemonic[0], buf);
+    LoadListBox(&boardOptions[THEMELIST], _("# no themes are defined"), -1, -1);
+    HighlightWithScroll(&boardOptions[THEMELIST], 0, nr);
+}
+
+void
 BoardOptionsProc ()
 {
    strncpy(oldPieceDir, appData.pieceDirectory, MSG_SIZ-1); // to see if it changed
+   ASSIGN(engineLine, "");
+   ASSIGN(nickName, "");
+   ASSIGN(engineMnemonic[0], "");
+   NamesToList(appData.themeNames, engineList, engineMnemonic, "");
    GenericPopUp(boardOptions, _("Board Options"), TransientDlg, BoardWindow, MODAL, 0);
 }
 
@@ -979,6 +1012,9 @@ SendString (char *p)
        snprintf(buf2 + (q-p), MSG_SIZ -(q-p), "%s%s", clickedWord, q+5);
         p = buf2;
     }
+    if(!strcmp(p, "$copy")) { // special case for copy selection
+        CopySomething(clickedWord);
+    } else
     if(!strcmp(p, "$chat")) { // special case for opening chat
         NewChat(clickedWord);
     } else
@@ -1707,7 +1743,8 @@ PromotionPopUp (char choice)
 //---------------------------- Chat Windows ----------------------------------------------
 
 static char *line, *memo, *chatMemo, *partner, *texts[MAX_CHAT], dirty[MAX_CHAT], *inputs[MAX_CHAT], *icsLine, *tmpLine;
-static int activePartner, hidden = 1;
+static int activePartner;
+int hidden = 1;
 
 void ChatSwitch P((int n));
 int  ChatOK P((int n));
@@ -1726,16 +1763,20 @@ WindowPlacement wpTextMenu;
 int
 ContextMenu (Option *opt, int button, int x, int y, char *text, int index)
 { // callback for ICS-output clicks; handles button 3, passes on other events
-  char *start, *end;
   int h;
   if(button == -3) return TRUE; // supress default GTK context menu on up-click
   if(button != 3) return FALSE;
-  start = end = text + index; // figure out what text was clicked
-  while(isalnum(*end)) end++;
-  while(start > text && isalnum(start[-1])) start--;
-  clickedWord[0] = NULLCHAR;
-  if(end-start >= 80) end = start + 80; // intended for small words and numbers
-  strncpy(clickedWord, start, end-start); clickedWord[end-start] = NULLCHAR;
+  if(index == -1) { // pre-existing selection in memo
+    strncpy(clickedWord, text, MSG_SIZ);
+  } else { // figure out what word was clicked
+    char *start, *end;
+    start = end = text + index;
+    while(isalnum(*end)) end++;
+    while(start > text && isalnum(start[-1])) start--;
+    clickedWord[0] = NULLCHAR;
+    if(end-start >= 80) end = start + 80; // intended for small words and numbers
+    strncpy(clickedWord, start, end-start); clickedWord[end-start] = NULLCHAR;
+  }
   click = !shellUp[TextMenuDlg]; // request auto-popdown of textmenu when we popped it up
   h = wpTextMenu.height; // remembered height of text menu
   if(h <= 0) h = 65;     // when not available, position w.r.t. top
@@ -1798,6 +1839,9 @@ IcsHist (int n, Option *opt, DialogClass dlg)
        if(hidden) BoardToTop();
        else PaneSwitch();
        break;
+      case 15:
+       NewChat(lastTalker);
+       break;
       case 14:
        for(chat=0; chat < MAX_CHAT; chat++) if(!chatPartner[chat][0]) break;
        if(chat < MAX_CHAT) ChatSwitch(chat + 1);
@@ -1858,7 +1902,7 @@ ChatOK (int n)
        // from here on it could be back-end
        if(line[strlen(line)-1] == '\n') line[strlen(line)-1] = NULLCHAR;
        SaveInHistory(line);
-       if(hidden) snprintf(buf, MSG_SIZ, "%s\n", line); else // command for ICS
+       if(hidden || !*chatPartner[activePartner]) snprintf(buf, MSG_SIZ, "%s\n", line); else // command for ICS
        if(!strcmp("whispers", chatPartner[activePartner]))
              snprintf(buf, MSG_SIZ, "whisper %s\n", line); // WHISPER box uses "whisper" to send
        else if(!strcmp("shouts", chatPartner[activePartner]))
@@ -1969,7 +2013,7 @@ ChatProc ()
     if(GenericPopUp(chatOptions, _("ICS Interaction"), ChatDlg, BoardWindow, NONMODAL, appData.topLevel))
        AddHandler(&chatOptions[CHAT_PARTNER], ChatDlg, 2), AddHandler(&chatOptions[CHAT_IN], ChatDlg, 2); // treats return as OK
     Show(&chatOptions[CHAT_PANE], hidden = 1); // hide
-    HardSetFocus(&chatOptions[CHAT_IN], 0);
+//    HardSetFocus(&chatOptions[CHAT_IN], 0);
     MarkMenu("View.OpenChatWindow", ChatDlg);
     CursorAtEnd(&chatOptions[CHAT_IN]);
 }
@@ -2875,10 +2919,9 @@ FileNamePopUp (char *label, char *def, char *filter, FileProc proc, char *openMo
 void
 ActivateTheme (int col)
 {
+    if(appData.overrideLineGap >= 0) lineGap = appData.overrideLineGap; else lineGap = defaultLineGap;
+    InitDrawingParams(strcmp(oldPieceDir, appData.pieceDirectory));
+    InitDrawingSizes(-1, 0);
+    DrawPosition(True, NULL);
 }
 
-char *
-Col2Text (int n)
-{
-    return NULL;
-}